# PARAM to give
#	$1 : date tag from sender (eg: 07161141 )
#	$2 : date tag from receiver (eg: 07161141,07161142 )
#	$3 : cctype ( utp / lp )
#	$4 : thput windows (in seconds)
#	$5 : IP of receiver (server)
#	$6 : bandwidth of bottleneck (in Mbps)
#
	#   e.g:    bash proc.sh 07161153 07161154 lp 1 10.0.0.2 0.5

tag1=$1
tag2=$2
trace="traces/$tag1/trace.$tag2.pcap"
std=$trace.std
evo=$trace.evo
ping_before="traces/$tag1/ping.$tag1.base.log"
ping_during="traces/$tag1/ping.$tag1.data.log"
cctype=$3
wnd=$4
ip=$5   # receiver
bw=$6

TCP_ports="5001 5002 5003 5004 5005"
#LBE_ports="5001 5002 5003 5004 5005"
LBE_ports="6001 6002 6003 6004 6005"

touch stat.evo.all

#==========================
# standardize the trace
#--------------------------
./bin/tcpdump.hack -n -r $trace -tt > $std

# data TX with TCP
proto="TCP"
data_TCP=$(for port in $TCP_ports ; do

	awk -v proto=$proto -v ip=$ip -v port=$port 'BEGIN { th=0 } ; { if (($6==ip)&&($7==port)&&($3==proto)) {th=th+$2} } ; END { print ip" "port" "proto" "th }' $std

done | awk 'BEGIN {sum=1} {sum=sum+$4} END {print sum}')

# data TX with LBE
if [ "$cctype" == "utp" ] ; then
	proto="UDP"
else
	proto="TCP"
fi
data_LBE=$(for port in $LBE_ports ; do

    awk -v proto=$proto -v ip=$ip -v port=$port 'BEGIN { th=0 } ; { if (($6==ip)&&($7==port)&&($3==proto)) {th=th+$2} } ; END { print ip" "port" "proto" "th }' $std

done | awk 'BEGIN {sum=1} {sum=sum+$4} END {print sum}')


#==========================
# compute the breakdown TCP%
#--------------------------
echo "data_TCP: "$data_TCP >> stat.evo.all
echo "data_LBE: "$data_LBE >> stat.evo.all
brk=`echo "scale=2; $data_TCP*100 / ($data_TCP+$data_LBE)" | bc`
echo "tcp%: "$brk >> stat.evo.all


#==========================
# compute Efficiency
#--------------------------
eff=`echo "scale=2; ($data_TCP+$data_LBE) / ($bw*60*1000000/8)" | bc`
echo "eff: "$eff >> stat.evo.all


#==========================
# compute queuing delay
#--------------------------
base_rtt=`tail -n 1 $ping_before | sed 's/\// /g' | awk '{print $7}'`
curr_rtt=`tail -n 1 $ping_during | sed 's/\// /g' | awk '{print $8}'`  # column: $8:avg , $9:max
queue=`echo "$curr_rtt - $base_rtt" | bc`
echo "queue: $queue" >> stat.evo.all


#==========================
# compute time evo of byte transmitted
#--------------------------
proto="TCP"
for port in $TCP_ports ; do
	awk -v wnd=$wnd -v proto=$proto -v ip=$ip -v port=$port 'BEGIN { data=0 } ; (NR==1) { split($1,time,"."); now=time[1] } { if (($6==ip)&&($7==port)&&($3==proto)) {split($1,curr,".") ; tot=tot+$2 ; if(curr[1]<(now+wnd)) {data=data+$2} else {print now" "data+$2; now=curr[1]; data=0} } } ' $std > $evo.$proto.$port
done
paste $evo.$proto.* | awk '{print $1" "$2" "$4" "$6" "$8" "$10}' > traces/$tag1/evo.$proto.all

if [ "$cctype" == "utp" ] ; then
	# use UTP log
	paste traces/$tag1/utp_recv.$tag2.*.log | awk '{print $1" "$2" "$5" "$8" "$11" "$14}' > traces/$tag1/evo.UTP.all
else
	proto="TCP"
	for port in $LBE_ports ; do
		awk -v wnd=$wnd -v proto=$proto -v ip=$ip -v port=$port 'BEGIN { data=0 } ; (NR==1) { split($1,time,"."); now=time[1] } { if (($6==ip)&&($7==port)&&($3==proto)) {split($1,curr,".") ; tot=tot+$2 ; if(curr[1]<(now+wnd)) {data=data+$2} else {print now" "data+$2; now=curr[1]; data=0} } } ' $std > $evo.LBE.$port
	done
	paste $evo.LBE.* | awk '{print $1" "$2" "$4" "$6" "$8" "$10}' > traces/$tag1/evo.LBE.all
fi

# TODO: properly align columns with TCP!
#paste $evo.*all | awk '{print $1" "$2" "$3" "$4" "$5" "$6" "$8" "$9" "$10" "$11" "$12}' > $evo.both

# plot the th-put evolution
DIR=traces/$tag1/$tag1.plot
mkdir $DIR
mv traces/$tag1/*evo* $DIR

if [ "$cctype" == "utp" ] ; then
	cp plot.evo.UTP.gp $DIR/plot.evo.gp
else
	cp plot.evo.LP.gp $DIR/plot.evo.gp
fi

cd $DIR
cat stat.evo.all
xgp plot.evo.gp

