#!/bin/bash -

# subfunctions
exportpath(){
	#read -a ans -n 1 -p "Export PATH? [y/N]:"
	#[[ "$ans" == "y" ]] && { export PATH=$PATH:$(pwd)/bin/; }
	export PATH=$PATH:$(pwd)/bin
	echo $PATH
	source ~/.bashrc
}

fixpermission(){
	chmod +x *.sh
	ls -l *.sh
}

gendata(){
	read -a ans -p "Data size (mbit)? [100]:"
	size=100
	[[ $ans =~ ^[0-9]+$ ]] && { size=$ans; } 
	dd if=/dev/zero of=tmp/${size}M.dat bs=1M count=$size
	cp tmp/${size}M.dat tmp/payload.send
}

setuphl(){
	#read -a ans -n 1 -p "Setup hard link? [y/N]:"
	#if [[ "$ans" == "y" ]]; then
	#read -a ans -n 1 -p "Role: Sender or Receiver? [s/r]:"
	#[[ "$ans" == "s" ]] && { ./setuphl.sh setup sender; }
	#[[ "$ans" == "r" ]] && { ./setuphl.sh setup receiver; }
	#fi
	sudo ifconfig eth2 10.0.0.3 netmask 255.255.255.0
	ifconfig eth2
}

setupsl(){
	#read -a ans -n 1 -p "Setup soft link? [y/N]:"
	#if [[ "$ans" == "y" ]]; then
	aqm="red"
	read -a ans -n 1 -p "AQM: FIFO, RED or SFQ (or Clean)? [f/R/s/c]:"
	if [[ "$ans" == "c" ]]; then
		./setupsl.sh clean
	else
		[[ "$ans" == "f" ]] && { aqm="fifo"; }
		[[ "$ans" == "s" ]] && { aqm="sfq"; }

		cap="0.5"
		read -a ans -p "Capacity: Default, DSL, DSL+ or FTTH? [0.5/1/10/100]:"
		[[ "$ans" == "1" ]] && { cap="1"; }		
		[[ "$ans" == "10" ]] && { cap="10"; }
		[[ "$ans" == "100" ]] && { cap="100"; }

		./setupsl.sh $aqm $cap
	fi
	#fi
}

startsender(){
	#read -a ans -n 1 -p "Start sender (make sure receiver is on)? [y/N]:"
	#[[ "$ans" == "y" ]] && { ./start_sender.sh }
	flow="1"
	read -a ans -p "Flows concurrent: [1/(int)]:"
	[[ "$ans" != "" ]] && { flow=$ans; }
	cctype="utp"
	read -a ans -p "CC Type: [UTP/lp]:"
	[[ "$ans" == "lp" ]] && { cctype=$ans; }
	ds="d"
	read -a ans -n 1 -p "Direct start (skip ip check)? [Y/n]:"
	[[ "$ans" == "n" ]] && { ds=""; }

	./start_sender.sh $flow $cctype $ds
}

startreceiver(){
	#read -a ans -n 1 -p "Start receiver? [y/N]:"
	#[[ "$ans" == "y" ]] && { ./start_receiver.sh }
	flow="1"
	read -a ans -p "Flows concurrent: [1/(int)]:"
	[[ "$ans" != "" ]] && { flow=$ans; }
	cctype="utp"
	read -a ans -p "CC Type: [UTP/lp]:"
	[[ "$ans" == "lp" ]] && { cctype=$ans; }
	ds="d"
	read -a ans -n 1 -p "Direct start (skip ip check)? [Y/n]:"
	[[ "$ans" == "n" ]] && { ds=""; }

	./start_receiver.sh $flow $cctype $ds
}

showenv(){
	echo -e "PATH:\n$PATH"
	echo "------------------------------------------"
	echo -e "Data file:\n$(du -h tmp/payload.send)"
	echo "------------------------------------------"
	echo -e "Sanity:\n$(ethtool -c eth0 | grep 'rx-usecs:'; ethtool -k eth0 | grep 'segmentation'; ethtool -k eth0 | grep 'receive'; )"
	echo "------------------------------------------"
	echo -e "TCP-LP:\n$(sysctl net.ipv4.tcp_allowed_congestion_control)"
	echo "------------------------------------------"
	echo -e "SetupHL:\ninet addr=$( ifconfig eth0 | grep 'inet addr' | awk -F: '{ print $2 }' | awk '{ print $1 }' )"
	echo "------------------------------------------"
	echo -e "SetupSL:\n$(tc qdisc show dev eth0)"
}

cleantmp(){
	rm -f tmp/*.log tmp/*.err tmp/*.out tmp/*.pcap tmp/*.recv
	echo "rm -f tmp/*.log tmp/*.err tmp/*.out tmp/*.pcap tmp/*.recv"
}

synctime(){
	echo "Help: install NTP: sudo apt-get install ntp"
	echo "Help: enable Broadcast and Client: /etc/ntp.conf"
	echo "Enable ntpd if possible..."
	ntpd
	echo "Print 6 seconds for testing:"
	p=6
	while [[ "$p" -gt 0 ]]; do date +%s; (( p = p - 1 )); sleep 1; done
}

collectdata(){
	echo "Retrieving data from sender and receiver..."
	scp paris@10.0.0.1:~/scripts.testbed/tmp/*.{pcap,log} ./tmp
	scp paris@10.0.0.2:~/scripts.testbed/tmp/*.{pcap,log} ./tmp
	echo "REMEMBER TO CLEAN TMP FOLDER ON CLIENT/SERVER!!!"
	#procdata()	
}

procdata(){
	echo "Processing data..."
	tagsender=$(echo tmp/ping.*.base.log | awk -F "[_.]" '{print $2}')
	tagreceiver=$(echo tmp/trace.*.pcap | awk -F "[_.]" '{print $2}')	
	echo "Detect tagsender=$tagsender tagreceiver=$tagreceiver"
	mkdir traces/$tagsender	
	mv tmp/*.log traces/$tagsender
	mv tmp/*.pcap traces/$tagsender
	mv tmp/*.std traces/$tagsender
	read -a ans -p "bandwidth: [(mbit)]:"
	[[ "$ans" != "" ]] && { bandwidth=$ans; } || { echo "bandwidth cannot be blank"; exit 0; }
	echo "./procstat.sh $tagsender $tagreceiver utp 1 10.0.0.2 $bandwidth"
	./procstat.sh $tagsender $tagreceiver utp 1 10.0.0.2 $bandwidth
}

plotdata(){
	#ATTENTION: ./proc.sh parameters are fixed here
	read -a ans -p "tagsender: [(int)]:"
	[[ "$ans" != "" ]] && { tagsender=$ans; } || { echo "tagsender cannot be blank"; exit 0; }
	[[ -d traces/$tagsender ]] || { echo "tagsender not found"; exit 0; }
	echo "Processing data..."
	tagreceiver=$(echo traces/$tagsender/trace.*.pcap | awk -F "[_.]" '{print $2}')	
	#echo "./proc2.sh $tagsender $tagreceiver utp 1 10.0.0.2 100"
	#./proc2.sh $tagsender $tagreceiver utp 1 10.0.0.2 100
	cd traces/$tagsender/$tagsender.plot
	cat stat.evo.all
	echo "../../../bin/xgp plot.evo.gp"
	../../../bin/xgp plot.evo.gp
	cd ../../..
}

# selector
dotask(){
	[[ "$ans" == "0" ]] && { fixpermission; }
#	[[ "$ans" == "1" ]] && { gendata; }
	[[ "$ans" == "1" ]] && { collectdata; }
	[[ "$ans" == "2" ]] && { setuphl; }
	[[ "$ans" == "3" ]] && { setupsl; }
	[[ "$ans" == "4" ]] && { startsender; }
	[[ "$ans" == "5" ]] && { startreceiver; }
	[[ "$ans" == "6" ]] && { procdata; }
	[[ "$ans" == "7" ]] && { cleantmp; }
	[[ "$ans" == "8" ]] && { synctime; }
	[[ "$ans" == "9" ]] && { plotdata; }
	[[ "$ans" == "q" ]] && { exit 0; }
}

# main procedure
# - shortcut
[[ $1 != *[!0-9]* && $1 -gt 0 && $1 -lt 9 ]] && { ans=$1; dotask; exit 0; }

while true; do
	echo "Make a wish:"
	echo " [0]  Permission   [1] Collect Data "
	echo " [2]  SetupHL      [x] SetupSL "
	echo " [x]  StartSender  [x] StartReceiver "
	echo " [6]  Proc Data    [7] Clean tmp "
	echo " [8]  Sync Time    [9] Plot Data"
	echo " [q] Quit"
	read -a ans -n 1
	echo -e "\n"	
	dotask
	echo -e "\n"
	continue
done

#TODO: using core controller to start sender and receiver
#TODO: auto clean tmp before each test


#add multi-flow, clean setupsl, utp_send/recv log file
#ping log file
#fix utp_send/recv log file with fflush
#change default link capacity to 0.5mbit, + integer conversion + red param check
#Add SFQ enabled
#change to multiple server mode to help parse pcap

#add LP enabled

#iperf: Locale.h, Locale.c, ReportDefault.c
#fix iperf no log (version pb)
#iperf print out at server side possible?

#proc enhancement

#NTP cause network crash...???
#add procedure for dsl test (ip problem)
#timestamp, broadcast / manual shift with indicators
#auto sync time, start, scp (change key first) and open file
#move $tag.log function to auto start script

#make sure restart send/recv before each run, unless connection refused
#utp_send/recv log write out only after file is transfered
#install: ethtool, NTP, gawk, ...
