#!/bin/bash -

# input check + restore
[[ "$1" == "clean" ]] && { tc qdisc del dev eth0 root; tc qdisc show dev eth0; exit 0; }
[[ "$1" == "fifo" || "$1" == "red" || "$1" == "sfq" ]] || { echo "Help: sudo ./setupsl.sh fifo|red|sfq [capacity(0.5mbit)]"; exit 1; }

# link check
src=$( ifconfig eth0 | grep 'inet addr' | awk -F: '{ print $2 }' | awk '{ print $1 }' )
dst="10.0.0.1"
[[ $src == "10.0.0.1" ]] && dst="10.0.0.2"
[[ $src == "10.0.0.1" || $src == "10.0.0.2" ]] || { echo "IP address not ready, run setuphl.sh first"; exit 1; }

# variable configure
cap=${2:-1}
red_prob=0.1
#red_cap=$((cap*1024))
red_cap=` echo "$cap*1024" | bc | sed 's/\.[0-9]*$//' ` 
red_ttl=50
red_avpkt=1500
red_max=$((red_cap*125*red_ttl/1000))
red_min=$((red_max/3))
[[ "$red_min" -lt "$red_avpkt" ]] && { red_min=$red_avpkt; }
red_burst=$(((red_min*2+red_max)/(3*red_avpkt)))
red_limit=$((red_max*8))

# main procedure
#2 - using ethtool to setup more reliable capacity [ONLY Support 10/100/1000 according to hardware and driver abailability]
#FOR TEST ONLY
#ethtool -s eth0 speed ${cap} duplex full autoneg off

# - reset 
tc qdisc del dev eth0 root

#1 - build tree
#tc qdisc add dev eth0 root handle 1: htb default 11

#dsl:
#tc class add dev eth0 parent 1: classid 1:1 htb rate 1mbit ceil 1mbit
#dsl+:
#tc class add dev eth0 parent 1: classid 1:1 htb rate 10mbit ceil 10mbit
#ftth:
#tc class add dev eth0 parent 1: classid 1:1 htb rate 100mbit ceil 100mbit

#1 - auto configure
#tc class add dev eth0 parent 1: classid 1:1 htb rate ${cap}mbit ceil ${cap}mbit

#2 - build tree
#tc qdisc add dev eth0 root handle 1: prio

# - add addr filter
#tc filter add dev eth0 protocol ip parent 1:0 prio 1 u32 match ip dst $dst flowid 1:1

# - add base "fifo" limitation and delay
#tc qdisc add dev eth0 parent 1:1 handle 11: netem delay 30ms limit 500

# - further aqm
if [[ "$1" == "fifo" ]]; then
	true
	#note: 500 pkt = 11760ms/5880ms/588ms/58ms delay with 0.5mbit/1mbit/10mbit/100mbit
elif [ "$1" == "red" ]; then	
	#FOR TEST ONLY
	tc qdisc add dev eth0 root handle 1: red limit $red_limit min $red_min max $red_max avpkt $red_avpkt burst $red_burst probability $red_prob bandwidth $red_cap	
	
	#auto configuration
	#tc qdisc add dev eth0 parent 11:1 handle 111: red limit $red_limit min $red_min max $red_max avpkt $red_avpkt burst $red_burst probability $red_prob bandwidth $red_cap

	#ns2 configuration	
	#tc qdisc add dev eth0 parent 11:1 handle 111: red limit 75000 min 3125 max 9375 avpkt 500 burst 10 probability 0.1 bandwidth 10240
		
	#not that bad for dsl 1mbit	
	#tc qdisc add dev eth0 parent 11:1 handle 111: red limit 150000 min 15000 max 150000 avpkt 1500 burst 10 probability 0.1 bandwidth 10240	

	#not that bad for dsl+ 10mbit	
	#tc qdisc add dev eth0 parent 11:1 handle 111: red limit 450000 min 15000 max 450000 avpkt 1500 burst 10 probability 0.01 bandwidth 10240	
elif [ "$1" == "sfq" ]; then
	tc qdisc add dev eth0 parent 11:1 handle 111: sfq perturb 10
fi

# result verify
[[ "$?" -ne 0 ]] && { echo "Error detected, restoring..."; tc qdisc del dev eth0 root; tc qdisc show dev eth0; exit 1; }
ethtool eth0 | grep 'Speed' | awk '{ print $1, $2 }'
tc qdisc show dev eth0
